"use strict";
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const flipper_common_1 = require("flipper-common");
const eventemitter3_1 = __importDefault(require("eventemitter3"));
const pluginKey_1 = require("./utils/pluginKey");
const immer_1 = require("immer");
const flipper_common_2 = require("flipper-common");
const flipper_plugin_core_1 = require("flipper-plugin-core");
const createServerAddOnControls_1 = require("./utils/createServerAddOnControls");
const isProduction_1 = __importDefault(require("./utils/isProduction"));
class AbstractClient extends eventemitter3_1.default {
    constructor(id, query, conn, logger, plugins, device, flipperServer) {
        super();
        this.connected = (0, flipper_plugin_core_1.createState)(false);
        this.sandyPluginStates = new Map();
        this.initializationPromise = new Promise((_resolve) => {
            this.resolveInitPromise = _resolve;
        });
        this.connected.set(!!conn);
        this.plugins = plugins ? plugins : new Set();
        this.backgroundPlugins = new Set();
        this.connection = conn;
        this.id = id;
        this.query = query;
        this.sdkVersion = query.sdk_version || 0;
        this.messageIdCounter = 0;
        this.logger = logger;
        this.activePlugins = new Set();
        this.device = device;
        this.flipperServer = flipperServer;
        this.serverAddOnControls = (0, createServerAddOnControls_1.createServerAddOnControls)(this.flipperServer);
    }
    isBackgroundPlugin(pluginId) {
        return this.backgroundPlugins.has(pluginId);
    }
    async init() {
        var _a;
        await this.loadPlugins('init');
        await Promise.all([...this.plugins].map(async (pluginId) => this.startPluginIfNeeded(await this.getPlugin(pluginId))));
        this.backgroundPlugins = new Set(await this.getBackgroundPlugins());
        this.backgroundPlugins.forEach((plugin) => {
            if (this.shouldConnectAsBackgroundPlugin(plugin)) {
                this.initPlugin(plugin);
            }
        });
        this.emit('plugins-change');
        (_a = this.resolveInitPromise) === null || _a === void 0 ? void 0 : _a.call(this, null);
    }
    // get the supported plugins
    async loadPlugins(_phase) {
        var _a;
        let response;
        try {
            response = await (0, flipper_common_2.timeout)(30 * 1000, this.rawCall('getPlugins', false), 'Fetch plugin timeout. Unresponsive client?');
        }
        catch (e) {
            console.warn('Failed to fetch plugin', e);
        }
        this.plugins = new Set((_a = response === null || response === void 0 ? void 0 : response.plugins) !== null && _a !== void 0 ? _a : []);
        console.info('AbstractClient.loadPlugins', this.query, [...this.plugins]);
        return this.plugins;
    }
    loadPlugin(plugin, initialState) {
        try {
            const pluginInstance = new flipper_plugin_core_1._SandyPluginInstance(this.serverAddOnControls, (0, flipper_plugin_core_1.getFlipperLib)(), plugin, this, (0, pluginKey_1.getPluginKey)(this.id, { serial: this.query.device_id }, plugin.id), initialState);
            pluginInstance.events.on('error', (message) => {
                const error = {
                    message,
                    name: 'Plugin Error',
                    stacktrace: '',
                };
                this.emit('error', error);
            });
            this.sandyPluginStates.set(plugin.id, pluginInstance);
        }
        catch (e) {
            console.error(`Failed to start plugin '${plugin.id}': `, e);
        }
    }
    startPluginIfNeeded(plugin) {
        if (plugin && !this.sandyPluginStates.has(plugin.id)) {
            this.loadPlugin(plugin);
        }
    }
    stopPluginIfNeeded(pluginId, _force = false) {
        const instance = this.sandyPluginStates.get(pluginId);
        if (instance) {
            instance.destroy();
            this.sandyPluginStates.delete(pluginId);
        }
    }
    // connection lost, but Client might live on
    disconnect() {
        this.sandyPluginStates.forEach((instance) => {
            instance.disconnect();
        });
        this.emit('close');
        this.connected.set(false);
    }
    // clean up this client
    destroy() {
        this.disconnect();
        this.plugins.forEach((pluginId) => this.stopPluginIfNeeded(pluginId, true));
        this.serverAddOnControls.unsubscribe();
    }
    // get the supported background plugins
    async getBackgroundPlugins() {
        if (this.sdkVersion < 4) {
            return [];
        }
        const data = await (0, flipper_common_2.timeout)(30 * 1000, this.rawCall('getBackgroundPlugins', false), 'Fetch background plugins timeout for ' + this.id);
        return data.plugins;
    }
    // get the plugins, and update the UI
    async refreshPlugins() {
        try {
            const oldBackgroundPlugins = this.backgroundPlugins;
            await this.loadPlugins('refresh');
            await Promise.all([...this.plugins].map(async (pluginId) => this.startPluginIfNeeded(await this.getPlugin(pluginId))));
            const newBackgroundPlugins = await this.getBackgroundPlugins();
            this.backgroundPlugins = new Set(newBackgroundPlugins);
            // diff the background plugin list, disconnect old, connect new ones
            oldBackgroundPlugins.forEach((plugin) => {
                if (!this.backgroundPlugins.has(plugin)) {
                    this.deinitPlugin(plugin);
                }
            });
            newBackgroundPlugins.forEach((plugin) => {
                if (!oldBackgroundPlugins.has(plugin) &&
                    this.shouldConnectAsBackgroundPlugin(plugin)) {
                    this.initPlugin(plugin);
                }
            });
            this.emit('plugins-change');
        }
        catch (e) {
            if (this.connected) {
                throw e;
            }
            else {
                console.warn(e);
            }
        }
    }
    onMessage(msg) {
        var _a, _b, _c, _d;
        if (typeof msg !== 'string') {
            return;
        }
        let rawData;
        try {
            rawData = (0, immer_1.freeze)(JSON.parse(msg), true);
        }
        catch (err) {
            console.error(`Invalid JSON: ${msg}`, 'clientMessage');
            return;
        }
        const data = rawData;
        const { id, method } = data;
        this.emit('flipper-debug-message', {
            device: (_a = this.device) === null || _a === void 0 ? void 0 : _a.displayTitle(),
            app: this.query.app,
            flipperInternalMethod: method,
            plugin: (_b = data.params) === null || _b === void 0 ? void 0 : _b.api,
            pluginMethod: (_c = data.params) === null || _c === void 0 ? void 0 : _c.method,
            payload: (_d = data.params) === null || _d === void 0 ? void 0 : _d.params,
            direction: 'toFlipper:message',
        });
        if (id == null) {
            const { error } = data;
            if (error != null) {
                console.error(`Error received from device ${method ? `when calling ${method}` : ''}: ${error.message} + \nDevice Stack Trace: ${error.stacktrace}`, 'deviceError');
                this.emit('error', error);
            }
            else if (method === 'refreshPlugins') {
                this.refreshPlugins();
            }
            else if (method === 'execute') {
                if (!data.params) {
                    throw new Error('expected params');
                }
                const params = data.params;
                const bytes = msg.length * 2; // string lengths are measured in UTF-16 units (not characters), so 2 bytes per char
                this.emit('bytes-received', params.api, bytes);
                if (bytes > 5 * 1024 * 1024) {
                    console.warn(`Plugin '${params.api}' received excessively large message for '${params.method}': ${Math.round(bytes / 1024)}kB`);
                }
                const handled = this.handleExecuteMessage(params);
                if (!handled && !(0, isProduction_1.default)()) {
                    console.warn(`Unhandled message ${params.api}.${params.method}`);
                }
            }
            // TODO: Warn about unknown method?
            return; // method === 'execute'
        }
    }
    handleExecuteMessage(params) {
        const pluginInstance = this.sandyPluginStates.get(params.api);
        if (!pluginInstance) {
            return false;
        }
        pluginInstance.receiveMessages([params]);
        return true;
    }
    onResponse(data, resolve, reject) {
        if (data.success) {
            resolve && resolve(data.success);
        }
        else if (data.error) {
            reject(data.error);
        }
    }
    rawCall(method, fromPlugin, params) {
        return new Promise(async (resolve, reject) => {
            var _a, _b, _c, _d;
            const id = this.messageIdCounter++;
            const metadata = {
                method,
                id,
                params,
            };
            const data = {
                id,
                method,
                params,
            };
            const plugin = params ? params.api : undefined;
            const baseErrorMessage = `Unable to send ${(_a = params === null || params === void 0 ? void 0 : params.method) !== null && _a !== void 0 ? _a : method} to ${(_b = params === null || params === void 0 ? void 0 : params.api) !== null && _b !== void 0 ? _b : 'FlipperCore'}`;
            console.debug(data, 'message:call');
            const mark = this.getPerformanceMark(metadata);
            performance.mark(mark);
            if (!this.connected.get()) {
                // TODO: display warning in the UI
                reject(new flipper_common_1.NoLongerConnectedToClientError());
                return;
            }
            if (!fromPlugin || this.isAcceptingMessagesFromPlugin(plugin)) {
                try {
                    const response = await this.connection.sendExpectResponse(data);
                    if (!fromPlugin || this.isAcceptingMessagesFromPlugin(plugin)) {
                        const logEventName = this.getLogEventName(data);
                        this.logger.trackTimeSince(mark, logEventName);
                        this.emit('bytes-received', plugin || 'unknown', response.length * 2);
                        this.onResponse(response, resolve, reject);
                        this.emit('flipper-debug-message', {
                            device: (_c = this.device) === null || _c === void 0 ? void 0 : _c.displayTitle(),
                            app: this.query.app,
                            flipperInternalMethod: method,
                            payload: response,
                            plugin,
                            pluginMethod: params === null || params === void 0 ? void 0 : params.method,
                            direction: 'toFlipper:response',
                        });
                    }
                }
                catch (error) {
                    reject(new Error(`${baseErrorMessage}, connection error: ${error}`));
                }
            }
            else {
                reject(new Error(`${baseErrorMessage}, client is not accepting messages for plugin ${plugin}`));
            }
            this.emit('flipper-debug-message', {
                device: (_d = this.device) === null || _d === void 0 ? void 0 : _d.displayTitle(),
                app: this.query.app,
                flipperInternalMethod: method,
                plugin: params === null || params === void 0 ? void 0 : params.api,
                pluginMethod: params === null || params === void 0 ? void 0 : params.method,
                payload: params === null || params === void 0 ? void 0 : params.params,
                direction: 'toClient:call',
            });
        });
    }
    isAcceptingMessagesFromPlugin(plugin) {
        return this.connection && (!plugin || this.activePlugins.has(plugin));
    }
    getPerformanceMark(data) {
        const { method, id } = data;
        return `request_response_${method}_${id}`;
    }
    getLogEventName(data) {
        const { method, params } = data;
        return params && params.api && params.method
            ? `request_response_${method}_${params.api}_${params.method}`
            : `request_response_${method}`;
    }
    initPlugin(pluginId) {
        this.activePlugins.add(pluginId);
        const instance = this.sandyPluginStates.get(pluginId);
        if (this.connected.get() && instance) {
            this.rawSend('init', { plugin: pluginId });
            instance.connect();
        }
    }
    deinitPlugin(pluginId) {
        this.activePlugins.delete(pluginId);
        const instance = this.sandyPluginStates.get(pluginId);
        instance === null || instance === void 0 ? void 0 : instance.disconnect();
        if (this.connected.get() && instance) {
            this.rawSend('deinit', { plugin: pluginId });
        }
    }
    rawSend(method, params) {
        var _a;
        const data = {
            method,
            params,
        };
        console.debug(data, 'message:send');
        if (this.connection) {
            this.connection.send(data);
        }
        this.emit('flipper-debug-message', {
            device: (_a = this.device) === null || _a === void 0 ? void 0 : _a.displayTitle(),
            app: this.query.app,
            flipperInternalMethod: method,
            payload: params,
            direction: 'toClient:send',
        });
    }
    call(api, method, fromPlugin, params) {
        return (0, flipper_common_1.reportPluginFailures)(this.rawCall('execute', fromPlugin, {
            api,
            method,
            params,
        }).catch((err) => {
            // We only throw errors if the connection is still alive
            // as connection-related ones aren't recoverable from
            // user code.
            if (this.connected.get()) {
                // This is a special case where we a send failed because of
                // a disconnect "mid-air". This can happen, for instance,
                // when you pull the plug from a connected phone. We can
                // still handle this gracefully.
                if (err.toString().includes('Socket closed unexpectedly')) {
                    console.warn(`Failed to call device due to unexpected disconnect: ${err}`);
                    this.disconnect();
                    return {};
                }
                throw err;
            }
            // This effectively preserves the previous behavior
            // of ignoring disconnection-related call failures.
            return {};
        }), `Call-${method}`, api);
    }
    send(api, method, params) {
        if (!(0, isProduction_1.default)()) {
            console.warn(`${api}:${method || ''} client.send() is deprecated. Please use call() instead so you can handle errors.`);
        }
        return this.rawSend('execute', { api, method, params });
    }
    async supportsMethod(api, method) {
        if (!this.connected.get()) {
            return false;
        }
        const response = await this.rawCall('isMethodSupported', true, {
            api,
            method,
        });
        return response.isSupported;
    }
}
exports.default = AbstractClient;
//# sourceMappingURL=AbstractClient.js.map