"use strict";
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
Object.defineProperty(exports, "__esModule", { value: true });
const flipper_plugin_core_1 = require("flipper-plugin-core");
const pluginKey_1 = require("../utils/pluginKey");
const js_base64_1 = require("js-base64");
const createServerAddOnControls_1 = require("../utils/createServerAddOnControls");
class BaseDevice {
    constructor(flipperServer, description, pluginErrorHandler) {
        this.pluginErrorHandler = pluginErrorHandler;
        this.isArchived = false;
        this.hasDevicePlugins = false; // true if there are device plugins for this device (not necessarily enabled)
        this.logListeners = new Map();
        this.crashListeners = new Map();
        this.connected = (0, flipper_plugin_core_1.createState)(true);
        // if imported, stores the original source location
        this.source = '';
        // TODO: ideally we don't want BasePlugin to know about the concept of plugins
        this.sandyPluginStates = new Map();
        this.deviceLogEventHandler = (payload) => {
            if (payload.serial === this.serial && this.logListeners.size > 0) {
                this.addLogEntry(payload.entry);
            }
        };
        this.crashLogEventHandler = (payload) => {
            if (payload.serial === this.serial && this.crashListeners.size > 0) {
                this.addCrashEntry(payload.crash);
            }
        };
        this.flipperServer = flipperServer;
        this.description = description;
        this.serverAddOnControls = (0, createServerAddOnControls_1.createServerAddOnControls)(this.flipperServer);
    }
    get isConnected() {
        return this.connected.get();
    }
    // operating system of this device
    get os() {
        return this.description.os;
    }
    // human readable name for this device
    get title() {
        return this.description.title;
    }
    // type of this device
    get deviceType() {
        return this.description.deviceType;
    }
    // serial number for this device
    get serial() {
        return this.description.serial;
    }
    // additional device specs used for plugin compatibility checks
    get specs() {
        var _a;
        return (_a = this.description.specs) !== null && _a !== void 0 ? _a : [];
    }
    // possible src of icon to display next to the device title
    get icon() {
        return this.description.icon;
    }
    supportsOS(os) {
        return os.toLowerCase() === this.os.toLowerCase();
    }
    displayTitle() {
        return this.connected.get() ? this.title : `${this.title} (Offline)`;
    }
    async exportState(idler, onStatusMessage, selectedPlugins) {
        const pluginStates = {};
        for (const instance of this.sandyPluginStates.values()) {
            if (selectedPlugins.includes(instance.definition.id) &&
                instance.isPersistable()) {
                pluginStates[instance.definition.id] = await instance.exportState(idler, onStatusMessage);
            }
        }
        return pluginStates;
    }
    toJSON() {
        return {
            os: this.os,
            title: this.title,
            deviceType: this.deviceType,
            serial: this.serial,
        };
    }
    addLogEntry(entry) {
        this.logListeners.forEach((listener) => {
            // prevent breaking other listeners, if one listener doesn't work.
            try {
                listener(entry);
            }
            catch (e) {
                console.error(`Log listener exception:`, e);
            }
        });
    }
    async startLogging() {
        this.flipperServer.on('device-log', this.deviceLogEventHandler);
    }
    stopLogging() {
        this.flipperServer.off('device-log', this.deviceLogEventHandler);
    }
    addLogListener(callback) {
        if (this.logListeners.size === 0) {
            this.startLogging();
        }
        const id = Symbol();
        this.logListeners.set(id, callback);
        return id;
    }
    removeLogListener(id) {
        this.logListeners.delete(id);
        if (this.logListeners.size === 0) {
            this.stopLogging();
        }
    }
    addCrashEntry(entry) {
        this.crashListeners.forEach((listener) => {
            // prevent breaking other listeners, if one listener doesn't work.
            try {
                listener(entry);
            }
            catch (e) {
                console.error(`Crash listener exception:`, e);
            }
        });
    }
    async startCrashWatcher() {
        this.flipperServer.on('device-crash', this.crashLogEventHandler);
    }
    stopCrashWatcher() {
        this.flipperServer.off('device-crash', this.crashLogEventHandler);
    }
    addCrashListener(callback) {
        if (this.crashListeners.size === 0) {
            this.startCrashWatcher();
        }
        const id = Symbol();
        this.crashListeners.set(id, callback);
        return id;
    }
    removeCrashListener(id) {
        this.crashListeners.delete(id);
        if (this.crashListeners.size === 0) {
            this.stopCrashWatcher();
        }
    }
    async navigateToLocation(location) {
        return this.flipperServer.exec('device-navigate', this.serial, location);
    }
    async installApp(appBundlePath, timeout) {
        return timeout
            ? this.flipperServer.exec({ timeout }, 'device-install-app', this.serial, appBundlePath)
            : this.flipperServer.exec('device-install-app', this.serial, appBundlePath);
    }
    async screenshot() {
        if (!this.description.features.screenshotAvailable || this.isArchived) {
            return;
        }
        return js_base64_1.Base64.toUint8Array(await this.flipperServer.exec('device-take-screenshot', this.serial));
    }
    async startScreenCapture(destination) {
        return this.flipperServer.exec('device-start-screencapture', this.serial, destination);
    }
    async stopScreenCapture() {
        return this.flipperServer.exec('device-stop-screencapture', this.serial);
    }
    async executeShell(command) {
        return this.flipperServer.exec('device-shell-exec', this.serial, command);
    }
    async sendMetroCommand(command) {
        return this.flipperServer.exec('metro-command', this.serial, command);
    }
    async forwardPort(local, remote) {
        return this.flipperServer.exec('device-forward-port', this.serial, local, remote);
    }
    async clearLogs() {
        return this.flipperServer.exec('device-clear-logs', this.serial);
    }
    supportsPlugin(plugin) {
        var _a, _b, _c, _d;
        let pluginDetails;
        if (plugin instanceof flipper_plugin_core_1._SandyPluginDefinition) {
            pluginDetails = plugin.details;
            if (!pluginDetails.pluginType && !pluginDetails.supportedDevices) {
                // TODO T84453692: this branch is to support plugins defined with the legacy approach. Need to remove this branch after some transition period when
                // all the plugins will be migrated to the new approach with static compatibility metadata in package.json.
                if (plugin instanceof flipper_plugin_core_1._SandyPluginDefinition) {
                    return (plugin.isDevicePlugin &&
                        ((_c = (_b = (_a = plugin.asDevicePluginModule()).supportsDevice) === null || _b === void 0 ? void 0 : _b.call(_a, this)) !== null && _c !== void 0 ? _c : false));
                }
                else {
                    return plugin.supportsDevice(this);
                }
            }
        }
        else {
            pluginDetails = plugin;
        }
        return (pluginDetails.pluginType === 'device' &&
            (!pluginDetails.supportedDevices ||
                ((_d = pluginDetails.supportedDevices) === null || _d === void 0 ? void 0 : _d.some((d) => (!d.os || d.os === this.os) &&
                    (!d.type || d.type === this.deviceType) &&
                    (d.archived === undefined || d.archived === this.isArchived) &&
                    (!d.specs || d.specs.every((spec) => this.specs.includes(spec)))))));
    }
    loadDevicePlugins(devicePlugins, enabledDevicePlugins, pluginStates) {
        if (!devicePlugins) {
            return;
        }
        const plugins = Array.from(devicePlugins.values()).filter((p) => enabledDevicePlugins === null || enabledDevicePlugins === void 0 ? void 0 : enabledDevicePlugins.has(p.id));
        for (const plugin of plugins) {
            this.loadDevicePlugin(plugin, pluginStates === null || pluginStates === void 0 ? void 0 : pluginStates[plugin.id]);
        }
    }
    loadDevicePlugin(plugin, initialState) {
        if (!this.supportsPlugin(plugin)) {
            return;
        }
        this.hasDevicePlugins = true;
        if (plugin instanceof flipper_plugin_core_1._SandyPluginDefinition) {
            try {
                const pluginInstance = new flipper_plugin_core_1._SandyDevicePluginInstance(this.serverAddOnControls, (0, flipper_plugin_core_1.getFlipperLib)(), plugin, this, 
                // break circular dep, one of those days again...
                (0, pluginKey_1.getPluginKey)(undefined, { serial: this.serial }, plugin.id), initialState);
                if (this.pluginErrorHandler) {
                    pluginInstance.events.on('error', this.pluginErrorHandler);
                }
                this.sandyPluginStates.set(plugin.id, pluginInstance);
            }
            catch (e) {
                console.error(`Failed to start device plugin '${plugin.id}': `, e);
            }
        }
    }
    unloadDevicePlugin(pluginId) {
        const instance = this.sandyPluginStates.get(pluginId);
        if (instance) {
            instance.destroy();
            this.sandyPluginStates.delete(pluginId);
        }
    }
    disconnect() {
        this.logListeners.clear();
        this.stopLogging();
        this.crashListeners.clear();
        this.stopCrashWatcher();
        this.connected.set(false);
    }
    destroy() {
        this.disconnect();
        this.sandyPluginStates.forEach((instance) => {
            instance.destroy();
        });
        this.sandyPluginStates.clear();
        this.serverAddOnControls.unsubscribe();
    }
}
exports.default = BaseDevice;
//# sourceMappingURL=BaseDevice.js.map