"use strict";
/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSandyPlugin = exports.wrapRequirePlugin = exports.createRequirePluginFunction = exports.checkDisabled = exports.checkGK = exports.getDynamicPlugins = exports.getLatestCompatibleVersionOfEachPlugin = exports.reportVersion = exports.isDevicePluginDefinition = exports.AbstractPluginInitializer = void 0;
const flipper_common_1 = require("flipper-common");
const flipper_common_2 = require("flipper-common");
const isPluginCompatible_1 = __importDefault(require("./utils/isPluginCompatible"));
const isPluginVersionMoreRecent_1 = __importDefault(require("./utils/isPluginVersionMoreRecent"));
const RenderHost_1 = require("./RenderHost");
const p_map_1 = __importDefault(require("p-map"));
class AbstractPluginInitializer {
    constructor() {
        this.gatekeepedPlugins = [];
        this.disabledPlugins = [];
        this.failedPlugins = [];
        this.loadedPlugins = [];
        this._initialPlugins = [];
    }
    async init() {
        this._initialPlugins = await this._init();
    }
    get initialPlugins() {
        return this._initialPlugins;
    }
    get requirePlugin() {
        return (0, exports.createRequirePluginFunction)(this.requirePluginImpl.bind(this));
    }
    async _init() {
        this.loadMarketplacePlugins();
        const uninstalledPluginNames = this.loadUninstalledPluginNames();
        const allLocalVersions = await this.loadAllLocalVersions(uninstalledPluginNames);
        const pluginsToLoad = await this.filterAllLocalVersions(allLocalVersions);
        const initialPlugins = await this.loadPlugins(pluginsToLoad);
        return initialPlugins;
    }
    loadMarketplacePlugins() { }
    loadUninstalledPluginNames() {
        return new Set();
    }
    async loadAllLocalVersions(uninstalledPluginNames) {
        const allLocalVersions = [...(await getDynamicPlugins())].filter((p) => !uninstalledPluginNames.has(p.name));
        return allLocalVersions;
    }
    async filterAllLocalVersions(allLocalVersions) {
        const flipperVersion = await this.getFlipperVersion();
        const loadedPlugins = getLatestCompatibleVersionOfEachPlugin(allLocalVersions, flipperVersion);
        this.loadedPlugins = loadedPlugins;
        const pluginsToLoad = loadedPlugins
            .map(reportVersion)
            .filter((0, exports.checkDisabled)(this.disabledPlugins))
            .filter((0, exports.checkGK)(this.gatekeepedPlugins));
        return pluginsToLoad;
    }
    async loadPlugins(pluginsToLoad) {
        const loader = this.requirePlugin(this.failedPlugins);
        const initialPlugins = (await (0, p_map_1.default)(pluginsToLoad, loader)).filter(flipper_common_1.notNull);
        return initialPlugins;
    }
}
exports.AbstractPluginInitializer = AbstractPluginInitializer;
function isDevicePluginDefinition(definition) {
    return definition.isDevicePlugin;
}
exports.isDevicePluginDefinition = isDevicePluginDefinition;
function reportVersion(pluginDetails) {
    (0, flipper_common_2.reportUsage)('plugin:version', {
        version: pluginDetails.version,
    }, pluginDetails.id);
    return pluginDetails;
}
exports.reportVersion = reportVersion;
function getLatestCompatibleVersionOfEachPlugin(plugins, flipperVersion) {
    const latestCompatibleVersions = new Map();
    for (const plugin of plugins) {
        if ((0, isPluginCompatible_1.default)(plugin, flipperVersion)) {
            const loadedVersion = latestCompatibleVersions.get(plugin.id);
            if (!loadedVersion ||
                (0, isPluginVersionMoreRecent_1.default)(plugin, loadedVersion, flipperVersion)) {
                latestCompatibleVersions.set(plugin.id, plugin);
            }
        }
    }
    return Array.from(latestCompatibleVersions.values());
}
exports.getLatestCompatibleVersionOfEachPlugin = getLatestCompatibleVersionOfEachPlugin;
async function getDynamicPlugins() {
    try {
        return await (0, RenderHost_1.getRenderHostInstance)().flipperServer.exec('plugins-load-dynamic-plugins');
    }
    catch (e) {
        console.error('Failed to load dynamic plugins', e);
        return [];
    }
}
exports.getDynamicPlugins = getDynamicPlugins;
const checkGK = (gatekeepedPlugins) => (plugin) => {
    try {
        if (!plugin.gatekeeper) {
            return true;
        }
        const result = (0, RenderHost_1.getRenderHostInstance)().GK(plugin.gatekeeper);
        if (!result) {
            gatekeepedPlugins.push(plugin);
        }
        return result;
    }
    catch (err) {
        console.error(`Failed to check GK for plugin ${plugin.id}`, err);
        return false;
    }
};
exports.checkGK = checkGK;
const checkDisabled = (disabledPlugins) => {
    const config = (0, RenderHost_1.getRenderHostInstance)().serverConfig;
    let enabledList = null;
    let disabledList = new Set();
    try {
        if (config.env.FLIPPER_ENABLED_PLUGINS) {
            enabledList = new Set(config.env.FLIPPER_ENABLED_PLUGINS.split(','));
        }
        disabledList = new Set(config.processConfig.disabledPlugins);
    }
    catch (e) {
        console.error('Failed to compute enabled/disabled plugins', e);
    }
    return (plugin) => {
        try {
            if (disabledList.has(plugin.name)) {
                disabledPlugins.push(plugin);
                return false;
            }
            if (enabledList &&
                !(enabledList.has(plugin.name) ||
                    enabledList.has(plugin.id) ||
                    enabledList.has(plugin.name.replace('flipper-plugin-', '')))) {
                disabledPlugins.push(plugin);
                return false;
            }
            return true;
        }
        catch (e) {
            console.error(`Failed to check whether plugin ${plugin.id} is disabled`, e);
            return false;
        }
    };
};
exports.checkDisabled = checkDisabled;
const createRequirePluginFunction = (requirePluginImpl) => (failedPlugins) => {
    return async (pluginDetails) => {
        try {
            const requirePluginImplWrapped = (0, exports.wrapRequirePlugin)(requirePluginImpl);
            const pluginDefinition = await requirePluginImplWrapped(pluginDetails);
            if (pluginDefinition &&
                isDevicePluginDefinition(pluginDefinition) &&
                pluginDefinition.details.pluginType !== 'device') {
                console.warn(`Package ${pluginDefinition.details.name} contains the device plugin "${pluginDefinition.title}" defined in a wrong format. Specify "pluginType" and "supportedDevices" properties and remove exported function "supportsDevice". See details at https://fbflipper.com/docs/extending/desktop-plugin-structure#creating-a-device-plugin.`);
            }
            return pluginDefinition;
        }
        catch (e) {
            failedPlugins.push([pluginDetails, e.message]);
            console.error(`Plugin ${pluginDetails.id} failed to load`, e);
            return null;
        }
    };
};
exports.createRequirePluginFunction = createRequirePluginFunction;
const wrapRequirePlugin = (requirePluginImpl) => (pluginDetails) => {
    (0, flipper_common_2.reportUsage)('plugin:load', {
        version: pluginDetails.version,
    }, pluginDetails.id);
    return (0, flipper_common_1.tryCatchReportPluginFailuresAsync)(() => requirePluginImpl(pluginDetails), 'plugin:load', pluginDetails.id);
};
exports.wrapRequirePlugin = wrapRequirePlugin;
const isSandyPlugin = (pluginDetails) => {
    return !!pluginDetails.flipperSDKVersion;
};
exports.isSandyPlugin = isSandyPlugin;
//# sourceMappingURL=plugins.js.map